@@ -32,7 +32,7 @@ import re
 import configparser
 from configparser import ConfigParser
 
-from typing import List, Dict, Tuple, Set, Optional
+from typing import List, Dict, Tuple, Set, Optional, Any
 
 from codecheck.check_result import CheckResult
 from codecheck.reporter import Reporter
@@ -54,6 +54,8 @@ ALL_CHECK_TYPES: List[str] = combine_value_lists(NAME_SUFFIX_TO_CHECK_TYPES)
 
 DEFAULT_CONF_FILE_NAME = 'codecheck.ini'
 
+NUM_REMAINING_CHECKS_TO_SHOW = 5
+
 
 def print_stats(description: str, d: Dict[str, int]) -> None:
     print("%s:\n    %s" % (
@@ -62,10 +64,16 @@ def print_stats(description: str, d: Dict[str, int]) -> None:
     ))
 
 
+if sys.version_info <= (3, 7):
+    CompiledRE = Any
+else:
+    CompiledRE = re.Pattern
+
+
 class CodeCheckConfig:
     mypy_config_path: str
     disabled_check_types: Set[str]
-    included_regex_list: Optional[List[re.Pattern]]
+    included_regex_list: Optional[List[CompiledRE]]
 
     def __init__(self) -> None:
         self.mypy_config_path = 'mypy.ini'
@@ -83,12 +91,12 @@ class CodeCheckConfig:
 
         def get_multi_line_regex_list(
                 section: configparser.SectionProxy,
-                field_name: str) -> List[re.Pattern]:
+                field_name: str) -> List[CompiledRE]:
             field_value = section.get(field_name)
             if field_value is None:
                 return None
             re_strings = field_value.strip().split('\n')
-            result: List[re.Pattern] = []
+            result: List[CompiledRE] = []
             for re_str in re_strings:
                 re_str = re_str.strip()
                 if not re_str:
@@ -141,6 +149,10 @@ class CodeChecker:
             '-v', '--verbose',
             action='store_true',
             help='Verbose output')
+        parser.add_argument(
+            '--detailed-progress',
+            action='store_true',
+            help='Show detailed progress information (even more verbose)')
         num_cpus = multiprocessing.cpu_count()
         parser.add_argument(
             '-j', '--parallelism',
@@ -276,7 +288,7 @@ class CodeChecker:
 
     def filter_with_glob_patterns(
             self, initial_list: List[str],
-            re_pattern_list: List[re.Pattern]) -> List[str]:
+            re_pattern_list: List[CompiledRE]) -> List[str]:
         filtered_list = [
             item for item in initial_list
             if any(re_pattern.match(item) for re_pattern in re_pattern_list)
@@ -361,11 +373,16 @@ class CodeChecker:
                 get_sys_path_entries_for_mypy())
 
         num_checks = len(check_inputs)
+        if self.args.verbose:
+            logging.info("Running %d checks", len(check_inputs))
+        pending_checks: Set[Tuple[str, str]] = set(check_inputs)
+
         with concurrent.futures.ThreadPoolExecutor(max_workers=args.parallelism) as executor:
             future_to_check_input = {
                 executor.submit(self.check_file, file_path, check_type): (file_path, check_type)
                 for (file_path, check_type) in check_inputs
             }
+            num_completed = 0
             for future in concurrent.futures.as_completed(future_to_check_input):
                 file_path, check_type = future_to_check_input[future]
                 try:
@@ -383,6 +400,19 @@ class CodeChecker:
                     else:
                         increment_counter(checks_by_result, 'failure')
                         is_success = False
+                num_completed += 1
+                pending_checks.remove((file_path, check_type))
+                if self.args.detailed_progress:
+                    remaining_checks_to_show = sorted(pending_checks)[:NUM_REMAINING_CHECKS_TO_SHOW]
+                    logging.info(
+                        "%d out of %d checks completed (%.1f%%), %d checks remaining, "
+                        "%d of them are: %s",
+                        num_completed,
+                        len(future_to_check_input),
+                        num_completed * 100.0 / len(future_to_check_input),
+                        len(pending_checks),
+                        len(remaining_checks_to_show),
+                        remaining_checks_to_show)
 
         if checks_by_dir:
             print_stats("Checks by directory (relative to repo root)", checks_by_dir)
